﻿//  RemoveNotes.jsx

/*
Иногда при внесении правки в текст важно контролировать, не уползла ли строка на другую полосу. 
Штатное решение: работая с одним документом, разделить экран пополам, чтобы на о дной половине отображались какие-то страницы, где перетекание недопустимо,
а на второй редактировать текст. Это хороший способ, и в дополнение к этому можно предложить вариант контроля перетекания с помощью скриптов.
    
Первый скрипт, AddNotes.jsx, помещает в начале и в конце каждой полосы метку, в которой сохраняется номер страницы и идентитфикатор фрейма, где помещена эта метка.
Формат записи:  М*S*F
где
М - это указатель места: В для обозначения верхней строки, или Н для обозначения нижней строки,
S - номер страницы
F - идентификатор фрейма

Перед началом размещения меток скрипт убирает имеющиеся в работе метки такого формата записи.
Независимо  от того, какая страница была активной, по завершении работы будет переход в начало файла и показ первой страницы на весь экран.

Второй скрипт, TestTextMoving.jsx, проверяет метки на соответствие их содержимого номеру страницы, на которой они стоят, и что они размещены в тех же фреймах.
Это позволит легко находить страницы, где возникло перетекание текста: при первом несоответствии выводится собщение о расхождении записанных данных с реальными,
и страница, где обнаружено расхождение, отображается на экране.

Третий скрипт, RemoveNotes.jsx, удаляет метки такого формата записи из работы.
*/

//   ©   Михаил Иванюшин 2020,  dotextok.ru  |  dotextok@gmail.com

var programTitul = "Маркировка строк";

// идея прогресс-бара взята отсюда: http://forums.adobe.com/message/3152162#3152162
var ProgressBar = function(title) { // ProgressBar
     var w = new Window('palette', title, {x:0, y:0, width:700, height:80},{closeButton: false}),
          st = w.add('statictext', {x:20, y:12, width:660, height:20});              
          pb = w.add('progressbar', {x:20, y:32, width:660, height:12}),
          ms = w.add('statictext', {x:20, y:52, width:660, height:20});
          st.justify = 'left';
          w.center();
     this.reset = function(msg,maxValue) {
          st.text = msg;
          pb.value = 0;
          pb.maxvalue = maxValue;
          pb.visible = maxValue;
          w.show();
          w.update();
      };
     this.info = function(msg) {
          ms.text = msg;
          w.show();
          w.update();
     };
     this.set = function(step) {pb.value = step;  w.update();};
     this.hit = function() {++pb.value;  w.update();};
     this.close = function() {w.close();};
} // ProgressBar

var repaint_value = 0; // 0 - перерисовка экрана выключена, это стандартное состояние

////>>>> темный экран / светлый экран >>>>>
// defSelColor: цвет фона кнопки, когда в её пространстве окажется курсор. Первые три числа: RGB, 0 - мин, 1 - макс.
// defBgColor:  цвет, который приобретёт фон кнопки, когда курсор покинет её пространство.
try {
    var uiBr = app.generalPreferences.uiBrightnessPreference;
    }
catch (e) {  // до СС темного варианта интерфейса не было
    uiBr = 0.6;
    }
if (uiBr == 1 || uiBr > 0.67) { 
    defBgColor = [.85,.85,.85, 1];  defSelColor = [.15,.15,.15, 1]; 
    }
else if (uiBr == 0.67 || uiBr > 0.33) { 
    defBgColor = [.7,.7,.7, 1]; defSelColor = [.3,.3,.3, 1]; 
    }
else if (uiBr == 0.33 || uiBr > 0) { 
    defBgColor = [.34,.34,.34, 1];  defSelColor = [.66,.66,.66, 1]; 
    }
else {
    /* if (uiBr == 0) */ defBgColor = [.2,.2,.2, 1];  defSelColor = [.8,.8,.8, 1];
    }
////>>>> темный экран / светлый экран >>>>>


/////////////////
var parseIntAppVersion = parseInt (app.version);
if (parseIntAppVersion < 8) { // app.version
	alert ("Этот скрипт предназначен для работы с текстом в InDesign CS6+.", programTitul);
    exit();
    } // app.version
var mySelection = app.selection[0];
if ((mySelection == null) || (app.selection[0].constructor.name == "TextFrame") || (app.selection[0].hasOwnProperty("texts") == false )) {
    alert("Должен быть выбран текст, чтобы определить статью. Все созданные ранее этим скриптом примечания будут удалены, и и на каждой странице этой статьи в начале первой строки и в конце последней будут вставлены примечания. В нём будет обозначение В или Н (верх или низ), номер страницы, где поставлена метка, и идентификатор фрейма.", programTitul);	
    exit();     
    }
if(app.documents[0].stories.length == 0) {
    alert("В открытом документе нет статей.", programTitul);	
    exit();    
    }
var noteInfo = [];
var myStory = mySelection.parentStory;
if (myStory.notes.length > 0) { // notes.length > 0
    nL = myStory.notes.length;
    var pBar = new ProgressBar(programTitul);
    pBar.reset("Удаление примечаний" , nL);    
    for (n = nL-1; n >=0; n--, pBar.hit()) { // n--
        while (noteInfo.length > 0) noteInfo.shift();  
        noteInfo = myStory.notes[n].texts[0].contents.split("*");
        if (noteInfo.length == 3) myStory.notes[n].remove();
        } // n--
    } // notes.length > 0
pBar.close();
exit(); 
